package eu.ztsh.garmin.mapbox

import android.content.res.Resources
import android.util.Log
import com.mapbox.api.directions.v5.models.RouteOptions
import com.mapbox.maps.EdgeInsets
import com.mapbox.maps.MapView
import com.mapbox.maps.Style
import com.mapbox.maps.plugin.gestures.gestures
import com.mapbox.maps.plugin.locationcomponent.location
import com.mapbox.maps.plugin.viewport.data.FollowPuckViewportStateOptions
import com.mapbox.maps.plugin.viewport.state.FollowPuckViewportState
import com.mapbox.maps.plugin.viewport.viewport
import com.mapbox.navigation.base.extensions.applyDefaultNavigationOptions
import com.mapbox.navigation.base.route.NavigationRoute
import com.mapbox.navigation.base.route.NavigationRouterCallback
import com.mapbox.navigation.base.route.RouterFailure
import com.mapbox.navigation.base.route.RouterOrigin
import com.mapbox.navigation.core.lifecycle.MapboxNavigationApp
import com.mapbox.navigation.ui.maps.location.NavigationLocationProvider
import com.mapbox.navigation.utils.internal.toPoint

class MapControl(val mapView: MapView, private val resources: Resources) {

    val navigationLocationProvider = NavigationLocationProvider()

    val routesRequestCallback = object : NavigationRouterCallback {
        override fun onRoutesReady(routes: List<NavigationRoute>, @RouterOrigin routerOrigin: String) {
            MapboxNavigationApp.current()?.setNavigationRoutes(routes)
        }

        override fun onFailure(reasons: List<RouterFailure>, routeOptions: RouteOptions) {
            Log.e(TAG, "onFailure: ")
        }

        override fun onCanceled(routeOptions: RouteOptions, @RouterOrigin routerOrigin: String) {
            Log.w(TAG, "onCanceled: ")
        }
    }



    fun init() {
        mapView.mapboxMap.loadStyle(Style.TRAFFIC_DAY) // TODO: base on sun position

        mapView.location.apply {
//            locationProvider = this.getLocationProvider()
//            setLocationProvider(navigationLocationProvider)
            puckBearingEnabled = true
            enabled = true
        }

        follow(true)
        setGestures(mapView)
    }

    fun follow(immediately: Boolean = false) {
        mapView.viewport.apply {
            // transition to followPuckViewportState with default transition
            val followPuckViewportState: FollowPuckViewportState = this.makeFollowPuckViewportState(
                FollowPuckViewportStateOptions.Builder()
//                .bearing(FollowPuckViewportStateBearing.Constant(0.0))
                    .padding(EdgeInsets(200.0 * resources.displayMetrics.density, 0.0, 0.0, 0.0))
                    .build()
            )
            if (immediately) {
                val immediateTransition = this.makeImmediateViewportTransition()
                this.transitionTo(followPuckViewportState, immediateTransition) { success ->
                    Log.d(TAG, "follow: $success")
                }
            } else {
                this.transitionTo(followPuckViewportState) { success ->
                    Log.d(TAG, "follow: $success")
                }
            }
        }
    }

    private fun setGestures(mapView: MapView) {
        mapView.gestures.apply {
            addOnMapClickListener { point ->
                mapView.location.isLocatedAt(point) { isPuckLocatedAtPoint ->
                    if (isPuckLocatedAtPoint) {
                        follow()
                    }
                }
                true
            }
            addOnMapLongClickListener { point ->
                MapboxNavigationApp.current()?.requestRoutes(
                    RouteOptions.builder()
                        .applyDefaultNavigationOptions()
                        .coordinatesList(mutableListOf(navigationLocationProvider.lastLocation!!.toPoint(), point))
                        .build(),
                    routesRequestCallback
                )
                true
            }
        }
    }

    companion object {

        const val TAG = "MAPCTRL"

    }
}