package eu.ztsh.garmin.mapbox

import android.util.Log
import com.mapbox.common.location.Location
import com.mapbox.navigation.core.MapboxNavigation
import com.mapbox.navigation.core.lifecycle.MapboxNavigationObserver
import com.mapbox.navigation.core.trip.session.LocationMatcherResult
import com.mapbox.navigation.core.trip.session.LocationObserver
import com.mapbox.navigation.ui.maps.camera.data.MapboxNavigationViewportDataSource

class NavigationObserver(private val mapControl: MapControl) : MapboxNavigationObserver {

    private val viewportDataSource = MapboxNavigationViewportDataSource(mapControl.mapView.mapboxMap)

    private val locationObserver = object : LocationObserver {

        override fun onNewLocationMatcherResult(locationMatcherResult: LocationMatcherResult) {
            mapControl.navigationLocationProvider.changePosition(
                location = locationMatcherResult.enhancedLocation,
                keyPoints = locationMatcherResult.keyPoints
            )
            viewportDataSource.onLocationChanged(locationMatcherResult.enhancedLocation)
            viewportDataSource.evaluate()
        }

        override fun onNewRawLocation(rawLocation: Location) {
            println()
        }

    }

    override fun onAttached(mapboxNavigation: MapboxNavigation) {
        mapboxNavigation.registerLocationObserver(locationObserver)
        Log.d(TAG, "Attached")
    }

    override fun onDetached(mapboxNavigation: MapboxNavigation) {
        mapboxNavigation.unregisterLocationObserver(locationObserver)
        Log.d(TAG, "Detached")
    }

    companion object {
        const val TAG = "MBOXOBS"
    }

}
